/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////


#ifndef __ISATCONVERTER_H__
#define __ISATCONVERTER_H__

#include "ABExport.h"
#include "../OdStreamBuf.h"
#include "../ModelerDefs.h"
#include "../OdArray.h"

/** Description:
    This class is the base class for classes that provide platform-dependent loading and saving
    of SAT files for Teigha.

    Library: TD_AcisBuilder
    {group:AcisBuilder_Classes}
*/
class AB_EXPORT ISATConverter
{
public:
  /** Description:
      Loads the specified SAT file. 
      Arguments:
      pStreamBuf (I) Pointer to the StreamBuf object from which the data are to be read.
      typeVer (O) Receives the type and version of the SAT file.
      standardSaveFlag (I) Controls the saving and restoring of use count data *in* the save file.
      Remarks:
      Returns true if and only if successful
  */
  virtual bool SatIn ( OdStreamBuf *pStreamBuf, AfTypeVer *typeVer, bool standardSaveFlag = true) = 0;
  /** Description:
      Saves the specified SAT file. 
      Arguments:
      pStreamBuf (I) Pointer to the StreamBuf object to which the data are to be written.
      typeVer (I) Type and version of the SAT file.
      standardSaveFlag (I) Controls the saving and restoring of use count data *in* the save file.
      Remarks:
      Returns true if and only if successful
  */
  virtual bool SatOut( OdStreamBuf *pStreamBuf, AfTypeVer  typeVer, bool standardSaveFlag = true) = 0;

  virtual void *getIBr( ) = 0;
};

AB_EXPORT bool ABBreakToSingleBody(OdStreamBuf *pStreamBuf, AfTypeVer *typeVer, bool standardSaveFlag, OdArray<ISATConverter*>& entitySet);

AB_EXPORT ISATConverter* ABCreateConverter();

AB_EXPORT void ABDeleteConverter(ISATConverter* pIC);

#endif // __ISATCONVERTER_H__
