/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////


#ifndef _INC_DDBRBREPEDGETRAVERSER_3F83F4F5003E_INCLUDED
#define _INC_DDBRBREPEDGETRAVERSER_3F83F4F5003E_INCLUDED

#include "../Br/BrEdge.h"
#include "../Br/BrBrep.h"
#include "../Br/BrTraverser.h"

#include "../TD_PackPush.h"

/** Description:
  This class is the interface class for BREP edge traversers.

  Library:
  TD_Br

  {group:OdBr_Classes}
*/
class ODBR_TOOLKIT_EXPORT OdBrBrepEdgeTraverser : public OdBrTraverser
{
public:
  /** Description:
    Sets this Traverser object to a specific BREP *edge* list.
    Arguments:
    brep (I) Owner of the *edge* list.
    
    Remarks:
    This Traverser object is set to the first element in the list.

    Returns odbrOK if successful, or an appropriate error code if not.
  */
  OdBrErrorStatus setBrep(const OdBrBrep& brep);

  /** Description:
    Sets this Traverser object to a specific BREP *edge* list
    and starting position.
    Arguments:
    edge (I) Defines the *edge* list and starting position.
  */
  OdBrErrorStatus setBrepAndEdge(const OdBrEdge& edge);

  /** Description:
    Sets the starting position of this Traverser object.
    Arguments:
    edge (I) Defines the starting position.
  */
  OdBrErrorStatus setEdge(const OdBrEdge& edge);

  /** Description:
    Returns the edge object at the current Traverser position.
  */
  OdBrEdge getEdge() const;
  /** Description:
    Returns the owner of the *edge* list associated with this Traverser object.
  */
  OdBrBrep getBrep() const;

  OdBrBrepEdgeTraverser();
};

#include "../TD_PackPop.h"

#endif /* _INC_DDBRBREPEDGETRAVERSER_3F83F4F5003E_INCLUDED */

