/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////



#ifndef _INC_DDBRFACE_3F82D5A203C8_INCLUDED
#define _INC_DDBRFACE_3F82D5A203C8_INCLUDED

#include "../Br/BrEntity.h"
#include "../Br/BrEnums.h"
#include "../Ge/GeSurface.h"
#include "../Ge/GeNurbSurface.h"

#include "../TD_PackPush.h"

class OdCmEntityColor;
class OdGeExternalBoundedSurface;

/** Description:
    This class is the interface class for BREP faces.

    Remarks:
    Faces are primary BREP elements with associated geometries: surfaces. 
    
    Library:
    TD_Br
    
    {group:OdBr_Classes}
*/
class ODBR_TOOLKIT_EXPORT OdBrFace : public OdBrEntity
{
public:

  /** Description:  
    Returns the underlying surface of this Face entity as an external OdGeNurbSurface.
    
    Arguments:
    nurb (O) Receives the NURBS surface.
  */
  OdBrErrorStatus getSurfaceAsNurb(OdGeNurbSurface& nurb) const;

  /** Description:  
    Returns the underlying surface of this Face entity as an OdGeSurface.
  */
  OdGeSurface* getSurface() const;

  /** Description:
    Returns the best match of the underlying surface of this Face entity to surfaces supported by OdGe.
    Arguments:
    surfaceType (O) Receives the surface type.
  */
  OdBrErrorStatus getSurfaceType(OdGe::EntityId& surfaceType) const;

  /** Description:
      Returns true if and only if the outside of the face is in
      the *direction* of the surface normal.
  */
  bool getOrientToSurface() const;
 /** Description:
    Returns the *color* of this Face entity.
   
    Remarks:
    Returns true if and only if this Face entity has a *color* associated with it.
    Arguments:
    color (O) Receives the *color*.
  */
  bool getColor(OdCmEntityColor &color) const;

  /** Description:
  Returns the *material handle* of this Face entity.

  Remarks:
  Returns true if and only if this Face entity has a *material* associated with it.
  Arguments:
  id (O) Receives the *id*.
  */
  bool getMaterialID(OdUInt64 &id) const;

  /** Description:
  Returns the *material mapper* of this Face entity.

  Remarks:
  Returns true if and only if this Face entity has a *material mapper* associated with it.
  Arguments:
  mx (O) Receives the *transformation matrix*.
  projection (O) Receives the *projection* flag.
  tiling (O) Receives the *tiling* flag.
  autoTransform (O) Receives the *auto transform* flag.
  */

  enum Projection
  {
    kInheritProjection  = 0, // Inherits *projection* from the current material's mapper.
    kPlanar             = 1, // Maps directly to XY coordinates.
    kBox                = 2, // Maps to planes perpendicular to major axes.
    kCylinder           = 3, // Maps to cylinder aligned with Z-axis.
    kSphere             = 4  // Maps to sphere aligned with Z-axis
  };

  enum Tiling
  {
    kInheritTiling    = 0, // Inherits *tiling* from the current material's mapper.
    kTile             = 1, // Repeats map along image axes.
    kCrop             = 2, // Crops map < 0.0 or > 1.0 on image axes.
    kClamp            = 3, // Clamps (stretches) map between 0.0 and 1.0 on image axes.
    kMirror           = 4  // Mirror the material map at every integer boundary.
  };

  enum AutoTransform
  {
    kInheritAutoTransform = 0x0, // Inherits automatic *transform* from the current material/s mapper.
    kNone                 = 0x1, // No automatic *transform*.
    kObject               = 0x2, // Adjusts the mapper *transform* to align with and fit the current object.
    kModel                = 0x4  // Multiples the mapper *transform* by the current block *transform*.
  };

  bool getMaterialMapper(OdGeMatrix3d &mx, Projection &projection, Tiling& tiling, AutoTransform &autoTransform) const;

  OdBrErrorStatus getSurfaceAsTrimmedNurbs(OdUInt32& numNurbs, OdGeExternalBoundedSurface**& ppNurbs) const;

  OdBrFace();

  ~OdBrFace();
};

#include "../TD_PackPop.h"

#endif /* _INC_DDBRFACE_3F82D5A203C8_INCLUDED */

