/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////



#ifndef _INC_DDBRLOOPEDGETRAVERSER_3F82E1FB0177_INCLUDED
#define _INC_DDBRLOOPEDGETRAVERSER_3F82E1FB0177_INCLUDED

#include "../Br/BrTraverser.h"

class OdBrEdgeLoopTraverser;
class OdGeNurbCurve3d;
class OdGeNurbCurve2d;
class OdGeCurve3d;
class OdGeCurve2d;
class OdBrLoop;
class OdBrEdge;


#include "../TD_PackPush.h"

/** Description:
    This class is the interface class for loop edge traversers.

    Library:
    TD_Br

    {group:OdBr_Classes}
*/
class ODBR_TOOLKIT_EXPORT OdBrLoopEdgeTraverser : public OdBrTraverser
{
public:

  /** Description:  
    Returns the underlying curve of the edge at the current Traverser position as an oriented OdGeCurve3d object.
  */
  OdGeCurve3d *getOrientedCurve() const;
  
  /** Description:  
    Returns the underlying curve of the edge at the current Traverser position as an OdGeNurbCurve3d object.

    Arguments:
    nurb (O) Receives the NURBS curve.
    
    Remarks:
    Returns true if and only if successful.
  */
  bool getOrientedCurveAsNurb(OdGeNurbCurve3d& nurb) const;

  /** Description:  
    Returns the underlying curve of the edge at the current Traverser position as a parametric OdGeCurve2d object.
  */
  OdGeCurve2d* getParamCurve() const;

  /** Description:  
    Returns the underlying curve of the edge at the current Traverser position as a parametric OdGeNurbCurve2d object.

    Arguments:
    nurb (O) Receives the NURBS curve.
    
    Remarks:
    Returns odbrOK if successful, or an appropriate error code if not.
  */
  OdBrErrorStatus getParamCurveAsNurb( OdGeNurbCurve2d& nurb ) const;

  /** Description:
    Returns true if and only if the orientation of the edge from vertex 1 to vertex 2 is the 
    same as the orientation of the loop parameterization.
  */
  bool getEdgeOrientToLoop() const;

  /** Description:
    Returns the edge object at the current Traverser position.
  */
  OdBrEdge getEdge() const;
  
 /** Description:
    Returns the loop of the *edge* list associated with this Traverser object.
  */
  OdBrLoop getLoop() const;


  /** Description:
    Sets the starting position of this Traverser object.
    Arguments:
    edge (I) Defines the starting position.
  */
  OdBrErrorStatus setEdge( const OdBrEdge& edge );
  
  /** Description:
    Sets this Traverser object to a specific *loop* *edge* list.
    Arguments:
    loop (I) Owner of the *edge* list.
    
    Remarks:
    This Traverser object is set to the first element in the list.

    Returns odbrOK if successful, or an appropriate error code if not.
  */
  OdBrErrorStatus setLoop( const OdBrLoop& loop );
  
  /** Description:
    Sets this Traverser object to a specific *edge* *loop* list
    and starting position.
    Arguments:
    edgeLoop (I) Defines the *edge* list and starting position.
  */
   OdBrErrorStatus setLoopAndEdge( const OdBrEdgeLoopTraverser &edgeLoop );

  OdBrLoopEdgeTraverser();
};

#include "../TD_PackPop.h"

#endif /* _INC_DDBRLOOPEDGETRAVERSER_3F82E1FB0177_INCLUDED */

