/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////


#ifndef __OD_GI_VIEWPORT_TRAITS_DATA__
#define __OD_GI_VIEWPORT_TRAITS_DATA__

#include "TD_PackPush.h"

#include "Gi/GiViewportTraits.h"

/** Description:
    Library: TD_Gi
    {group:OdGi_Classes}
*/
class OdGiViewportTraitsData
{
  OdDbStub*                               m_backgroundId;
  bool                                    m_bDefLightOn;
  OdGiViewportTraits::DefaultLightingType m_lightType;
  OdCmEntityColor                         m_colorAmbient;
  double                                  m_dBrightness;
  double                                  m_dContrast;
  OdDbStub*                               m_renderEnvId;
  OdDbStub*                               m_renderSettings;
  OdDbStub*                               m_visualStyle;

public:

  OdGiViewportTraitsData()
    : m_backgroundId(NULL)
    , m_bDefLightOn(false)
    , m_lightType(OdGiViewportTraits::kOneDistantLight)
    , m_colorAmbient()
    , m_dBrightness(50.0)
    , m_dContrast(50.0)
    , m_renderEnvId(NULL)
    , m_renderSettings(NULL)
    , m_visualStyle(NULL)
  {}
  
  OdGiDrawable::DrawableType type() const { return OdGiDrawable::kViewport; }

  void setBackground(OdDbStub* backgroundId)
  {
    m_backgroundId = backgroundId;
  }
  OdDbStub* background() const
  {
    return m_backgroundId;
  }

  void setDefaultLightingOn(bool on)
  {
    m_bDefLightOn = on;
  }
  bool isDefaultLightingOn() const
  {
    return m_bDefLightOn;
  }

  void setDefaultLightingType(OdGiViewportTraits::DefaultLightingType typ)
  {
    m_lightType = typ;
  }
  OdGiViewportTraits::DefaultLightingType defaultLightingType() const
  {
    return m_lightType;
  }

  void setAmbientLightColor(const OdCmEntityColor& clr)
  {
    m_colorAmbient = clr;
  }
  OdCmEntityColor ambientLightColor() const
  {
    return m_colorAmbient;
  }

  void setBrightness(double brightness)
  {
    m_dBrightness = brightness;
  }
  double brightness() const
  {
    return m_dBrightness;
  }

  void setContrast(double contrast)
  {
    m_dContrast = contrast;
  }
  double contrast() const
  {
    return m_dContrast;
  }

  void setRenderEnvironment(OdDbStub* renderEnvId)
  {
    m_renderEnvId = renderEnvId;
  }
  OdDbStub* renderEnvironment() const
  {
    return m_renderEnvId;
  }

  void setRenderSettings(OdDbStub* renderSettingsId)
  {
    m_renderSettings = renderSettingsId;
  }
  OdDbStub* renderSettings() const
  {
    return m_renderSettings;
  }

  void setVisualStyle(OdDbStub* visualStyleId)
  {
    m_visualStyle = visualStyleId;
  }
  OdDbStub* visualStyle() const
  {
    return m_visualStyle;
  }
};                          

#include "TD_PackPop.h"

#endif // __OD_GI_VIEWPORT_TRAITS_DATA__
