/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////




#ifndef _OD_PLATFORM_SETTINGS_H_
#define _OD_PLATFORM_SETTINGS_H_


#include <stddef.h>
#include <stdlib.h>
#include <limits.h>

#define TD_UNICODE


#if defined(_MSC_VER)
// warning C4290: C++ Exception Specification ignored
#pragma warning ( disable : 4290 )
// warning C4514: ... : unreferenced inline function has been removed
#pragma warning ( disable : 4514 )
// identifier was truncated to '255' characters in the debug information
#pragma warning ( disable : 4786 )
// class 'NAME' needs to have dll-interface to be used by clients of class NAME
#pragma warning ( disable : 4251 )
// copy constructor could not be generated
#pragma warning ( disable : 4511 ) 
// assignment operator could not be generated
#pragma warning ( disable : 4512 ) 
// alignment changed after including header
#pragma warning ( disable : 4103 ) 

#ifndef _DEBUG
#pragma warning ( disable : 4100 ) // unreferenced formal parameter
#pragma warning ( disable : 4702 ) // unreachable code
#pragma warning ( disable : 4710 ) // not inlined
#endif

#define ODRX_ABSTRACT __declspec(novtable)

#if (_MSC_VER > 1310)
#define ODRX_OVERRIDE override
#define ODRX_SEALED sealed
#define ODRX_SEALED_VIRTUAL virtual
#pragma warning ( disable : 4481 ) // nonstandard extension used: override specifier 'sealed'
#else
#define ODRX_OVERRIDE
#define ODRX_SEALED
#define ODRX_SEALED_VIRTUAL
#endif 

#else

#define ODRX_ABSTRACT
#define ODRX_OVERRIDE
#define ODRX_SEALED
#define ODRX_SEALED_VIRTUAL

#endif  // _MSC_VER

#define TD_FUNC __FUNCTION__

#ifdef _MSC_VER                                    ///////////////////////// Microsoft Visual C++

  //#define TD_USING(a)
  #if _MSC_VER <= 1200  
    #undef TD_FUNC
    #define TD_FUNC OD_T("")
  #endif

  #if _MSC_VER > 1200  && !defined(_WIN32_WCE) // || _MSC_VER = 1   // 1200 == VC6
    #define OD_TYPENAME typename
    #define OD_STD_ALLOCATOR_AS_IS
  #endif

  #if _MSC_VER >= 1400
    #pragma warning ( disable : 4996 )
    #ifndef _CRT_SECURE_NO_DEPRECATE
      #define _CRT_SECURE_NO_DEPRECATE
    #endif
  #endif

  #if defined(_IA64_)
    #define TD_STRICT_ALIGNMENT
  #endif

  #ifdef _WIN32_WCE                                ///////////////////////// Windows CE

    #define OD_TYPENAME typename
    #define OD_STRING_FNS
    #define OD_NO_GETENV

    extern void *bsearch(const void *key, const void *base, size_t numeles, size_t widthele, int(__cdecl *cfunc)(const void *elem1, const void *elem2));

    #ifdef ARM
      #define TD_STRICT_ALIGNMENT
    #endif // ARM

  #endif // _WIN32_WCE

  #define OD_LOGB _logb

#elif defined(__sun)                               ///////////////////////// Solaris

  #define OD_STRING_FNS
  #define OD_STD_ALLOCATOR
  #define ODA_UNIXOS

  #ifndef __GNUC__
    #undef TD_FUNC
    #define TD_FUNC OD_T("")
  #endif


  #define OD_TYPENAME typename

  #ifndef __GNUC__
    #define OD_TYPENAME2 typename
    #define OD_LINKEDARRAY_SCOPE OdLinkedArray<T, A>::
  #endif

  #if defined(sparc)
    #define ODA_BIGENDIAN
  #endif

  #if defined(__GNUC__)
    #define TD_USING(a) using a
    #define ODA_NEED_TEMP_USING
  #endif
#elif defined(ANDROID)

  #define TD_STRICT_ALIGNMENT
  #define OD_STRING_FNS
  #define ODA_BIGENDIAN
  #define OD_TYPENAME typename
  #define ODA_UNIXOS
  #define ODA_NEED_TEMP_USING
  #define OD_STD_ALLOCATOR_AS_IS

#elif defined(__APPLE__) && defined(__GNUC__)    ///////////////////////// Mac OS/X

  #ifdef __IPHONE_OS_VERSION_MIN_REQUIRED        // iOS devices require aligned doubles
    #define TD_STRICT_ALIGNMENT
  #endif

  #define OD_STRING_FNS
#if defined(__BIG_ENDIAN__) || !defined(__LITTLE_ENDIAN__)
  #define ODA_BIGENDIAN
#endif
  #define OD_TYPENAME typename
  #define ODA_UNIXOS
  #define ODA_NEED_TEMP_USING
  #define OD_STD_ALLOCATOR_AS_IS

#elif defined(sgi)                                 ///////////////////////// SGI

  #define OD_STRING_FNS
  #define ODA_BIGENDIAN
  #define OD_STD_ALLOCATOR
  #ifdef ODA_SGI_721 
    #define OD_TYPENAME 
  #else 
    #define OD_TYPENAME typename 
  #endif 
  #define ODA_UNIXOS
  #undef TD_FUNC
  #define TD_FUNC OD_T("")


#elif defined(__hpux)                              ///////////////////////// HP RISC/Itanium

  #define OD_STRING_FNS
  #define ODA_BIGENDIAN
  #define OD_STD_ALLOCATOR
  #define ODA_NEED_TEMP_USING
  #define ODA_FORMAT
  #define ODA_UNIXOS

  // Requires typename keyword in some contexts, which are illegal with other compilers.
  #define OD_TYPENAME typename

  #define TD_USING(a) using a;

#elif defined(_AIX)                                //////////////////////// IBM AIX

  #define OD_STRING_FNS
  #define ODA_BIGENDIAN
  #define OD_STD_ALLOCATOR_AS_IS
  #define ODA_NEED_TEMP_USING
  // Requires typename keyword in some contexts, which are illegal with other compilers.
  #define OD_TYPENAME typename
  #define OD_TYPENAME3 typename
  #define ODA_UNIXOS

#elif defined(__BCPLUSPLUS__)|| defined(__BORLANDC__) //////////////////////// Borland C++

  #undef TD_FUNC
  #define TD_FUNC __FUNC__

  #define OD_STD_ALLOCATOR_AS_IS
  #define ODA_GCC_2_95
  #define OD_BSEARCH std::bsearch
  #define OD_LOGB _logb

#elif defined(__linux__)|| defined(__FreeBSD__)  || defined(__MVS__)

  #define OD_STRING_FNS
  #define OD_TYPENAME typename
  #define ODA_UNIXOS
  #define ODA_NEED_TEMP_USING
  #define OD_STD_ALLOCATOR_AS_IS
  // Linux ppc requires big endian byte ordering
  #if defined(__BIG_ENDIAN__) || defined(__s390__)
    #define ODA_BIGENDIAN
  #endif

#elif defined(__MWERKS__)                          //////////////////////// Metrowerks Codewarrior

  #include <TargetConditionals.h>

  #if TARGET_OS_MAC
    #define ODA_BIGENDIAN
    #define OD_STRING_FNS
    #define OD_NO_GETENV
  #endif

#elif defined(__CYGWIN32__) || defined(__MINGW32__) //////////////////////// MINGW (Cygwin)

  #define ODA_UNIXOS
  #define OD_TYPENAME typename
  #define OD_LOGB ilogb
  #define OD_STRING_FNS
  #define ODA_NEED_TEMP_USING

#else                                              //////////////////////// Unsupported

  #error "Unsupported Platform/Compiler"

#endif


#ifndef OD_LOGB
  #define OD_LOGB logb
#endif



#define OD_T(x) ((const OdChar*)__OD_T(x))
#define __OD_T(x) L ## x

#if (defined(WIN32) || defined(WIN64) || defined(_WIN32_WCE)) && !defined(__MINGW32__)
#define OD_CONCAT(a, b) __OD_T(a) ## __OD_T(b)
#if defined(__BCPLUSPLUS__) && (__BCPLUSPLUS__ >= 1408)
  #define DDLL(a) a##ULL
#else
  #define DDLL(a) a
#endif
#else
  #if defined(__sun) || defined(sgi) || defined(__hpux) || defined(_AIX)
    #define OD_CONCAT(a, b) L##a L##b
  #else
    #define OD_CONCAT(a, b) L##a b
  #endif
  #define DDLL(a) a##ULL
#endif

#if defined(_NATIVE_WCHAR_T_DEFINED) && (!defined(_WIN32_WCE) || _MSC_VER < 1300)
/*/
typedef unsigned short OdChar;
typedef unsigned short OdUChar;
/*/
typedef wchar_t OdChar;
typedef wchar_t OdUChar;
#define NATIVE_ODCHAR
//*/
#else
typedef wchar_t OdChar;
typedef wchar_t OdUChar;
#endif


//#define OdBool_is_bool 1
#if defined(OdBool_is_bool)
  typedef bool OdBool;
  enum { kFalse = false, kTrue = true };
#else
  typedef int OdBool;
  enum { kFalse = 0, kTrue = 1 }; 
#endif

#define OD_BOOL(a) ((a) ? kTrue : kFalse)

//#define OdSize_is_size_t 1
#if defined(OdSize_is_size_t)
  typedef size_t OdSize;
#else
  typedef int OdSize;
#endif

#if (defined(WIN32) || defined(WIN64))
#define ODCHAR_IS_INT16LE 1
#else
#define ODCHAR_IS_INT16LE 0
#endif

typedef signed char OdInt8;
typedef short OdInt16;
typedef int OdInt;

typedef unsigned char OdUInt8;
typedef unsigned short OdUInt16;

#if   UINT_MAX == 0xFFFFFFFFUL
#define OD_SIZEOF_INT  4
#elif UINT_MAX > 0xFFFFFFFFU && UINT_MAX == 0xFFFFFFFFFFFFFFFFU
#error "8 byte size of `int' type unsupported!"
#else
#error "Unsupported number of *bytes* in `int' type!"
#endif

#if   ULONG_MAX == 0xFFFFFFFFUL
#define OD_SIZEOF_LONG  4
#elif (ULONG_MAX > 0xFFFFFFFFU && ULONG_MAX == 0xFFFFFFFFFFFFFFFFU) || (defined(sparc) && defined(_LP64))
#define OD_SIZEOF_LONG  8
#else
#error "Unsupported number of *bytes* in `long' type!"
#endif

#if OD_SIZEOF_LONG == 4
typedef long OdInt32;
typedef unsigned long OdUInt32;
#else // assumes 4-byte int type
typedef int OdInt32;
typedef unsigned int OdUInt32;
#endif

#if defined(_IA64_) || (defined(_MSC_VER) && (_MSC_VER >= 1020)) || defined(__BORLANDC__) || defined(__BCPLUSPLUS__)
#define BUILTIN_INT64 __int64
#define OD_ATOI64 _atoi64
#define PERCENT18LONG "%18I64d\r\n"
#define HANDLEFORMAT "%I64X\r\n"

#elif OD_SIZEOF_LONG == 8
#define BUILTIN_INT64 long
#define OD_ATOI64 atol
#define PERCENT18LONG  "%18ld\r\n"
#define HANDLEFORMAT  "%lX\r\n"

#elif defined(__MWERKS__) || defined(__GNUC__) || defined(sgi) || defined(_AIX) || defined(__sun) || defined(__hpux)
#define BUILTIN_INT64 long long
#define OD_ATOI64 atoll
#if (defined(__hpux) && !defined(atoll) && defined(__cplusplus)) || defined(_AIX_SOURCE)
inline long long atoll(const char *instr)
{
  long long retval;
  int i;

  retval = 0;
  for (; *instr; instr++) {
    retval = 10*retval + (*instr - '0');
  }
  return retval;
}
#endif
#define PERCENT18LONG "%18lld\r\n"
#define HANDLEFORMAT "%llX\r\n"
#endif

#ifdef BUILTIN_INT64
typedef BUILTIN_INT64 OdInt64;
typedef unsigned BUILTIN_INT64 OdUInt64;
#else
#include "Int64.h"
#define OD_ATOI64 atoi // TODO
#error ("sprinf can't process this without built-in in64 type")
#define PERCENT18LONG "%18d\r\n" // TODO
#define HANDLEFORMAT "%X\r\n"
#endif

// C99 definitions for int64 portable printf format
#ifdef _MSC_VER
#ifndef PRId64W
#define PRId64W L"%I64d"
#endif
#ifndef PRIu64W
#define PRIu64W L"%I64u"
#endif
#ifndef PRIx64W
#define PRIx64W L"%I64x"
#endif
#ifndef PRIX64W
#define PRIX64W L"%I64X"
#endif
#ifndef PRId64
#define PRId64 "%I64d"
#endif
#ifndef PRIu64
#define PRIu64 "%I64u"
#endif
#ifndef PRIx64
#define PRIx64 "%I64x"
#endif
#ifndef PRIX64
#define PRIX64 "%I64X"
#endif
#else // ifdef _MSC_VER
#ifndef PRId64W
#define PRId64W L"%lld"
#endif
#ifndef PRIu64W
#define PRIu64W L"%llu"
#endif
#ifndef PRIx64W
#define PRIx64W L"%llx"
#endif
#ifndef PRIX64W
#define PRIX64W L"%llX"
#endif
#ifndef PRId64
#define PRId64 "%lld"
#endif
#ifndef PRIu64
#define PRIu64 "%llu"
#endif
#ifndef PRIx64
#define PRIx64 "%llx"
#endif
#ifndef PRIX64
#define PRIX64 "%llX"
#endif
#endif //ifdef _MSC_VER


typedef void * VoidPtr;
//typedef ptrdiff_t OdIntPtr;
#if (OD_SIZEOF_LONG == 4) && !defined(_WIN64) && !defined(__hpux) && !defined(_AIX)
#define OD_INTPTR_UNIQUE
#endif

#if !defined(_WIN64) && !defined (_AC64)

typedef __w64 long          OdLongPtr;
typedef __w64 unsigned long OdULongPtr;
//
typedef __w64 int           OdIntPtr;
typedef __w64 unsigned int  OdUIntPtr;

#else

typedef __int64             OdLongPtr;
typedef unsigned __int64    OdULongPtr;
typedef __int64             OdIntPtr;
typedef unsigned __int64    OdUIntPtr;

#endif

typedef OdIntPtr OdGsMarker; // Integer able to store a pointer (32 or 64 depending on platform)

#if defined (_WIN64)
#define OD_SIZEOF_PTR 8
#else
// This may need to be adjusted
#define OD_SIZEOF_PTR OD_SIZEOF_LONG
#endif

#define OdIntToPtr(arg)   ((void*)((OdIntPtr)(arg)))

#define OdPtrToInt32(arg)   ((OdInt32)((OdIntPtr)(arg)))

#define OdIntToGsMarker(arg) ((OdGsMarker)((OdIntPtr)(arg)))

#define OdGsMarkerToInt32(arg) OdPtrToInt32(arg)

//////////////////////////////////////////////////////////////////////////
//  Non-MSVC dynamic library loading.

#ifdef  _TOOLKIT_IN_DLL_

  #if defined(_MSC_VER) || defined(__BORLANDC__)

    #pragma warning( disable: 4275 4251 )
    #define OD_DLL_EXPORT __declspec(dllexport)
    #define OD_DLL_IMPORT __declspec(dllimport)

  #elif defined(__GNUC__) && defined(__APPLE__) // GNU compiler MacOS X (xCode)

    #define OD_DLL_EXPORT __attribute__((visibility("default")))
    #define OD_DLL_IMPORT __attribute__((visibility("default")))

  #else // unix, linex and others

    #define OD_DLL_EXPORT __attribute__((visibility("default")))
    #define OD_DLL_IMPORT __attribute__((visibility("default")))

  #endif

  #define OD_EXTERN  extern

  #if defined(_MSC_VER)
    #define OD_TOOLKIT_EXPORT OD_DLL_EXPORT
    #define OD_STATIC_EXPORT
    #define OD_TOOLKIT_IMPORT 
    #define OD_STATIC_IMPORT OD_DLL_IMPORT
  #elif defined(__BORLANDC__)
    #define OD_TOOLKIT_EXPORT  OD_DLL_EXPORT
    #define OD_STATIC_EXPORT
    #define OD_TOOLKIT_IMPORT  OD_DLL_IMPORT
    #define OD_STATIC_IMPORT
  #else
    #define OD_TOOLKIT_EXPORT  OD_DLL_EXPORT
    #define OD_STATIC_EXPORT
    #define OD_TOOLKIT_IMPORT
    #define OD_STATIC_IMPORT   OD_DLL_IMPORT
  #endif

#else   // of _TOOLKIT_IN_DLL_

  #define OD_TOOLKIT_EXPORT
  #define OD_STATIC_EXPORT
  #define OD_TOOLKIT_IMPORT
  #define OD_STATIC_IMPORT

  #define OD_DLL_EXPORT
  #define OD_DLL_IMPORT
  #define OD_EXTERN

#endif  // of _TOOLKIT_IN_DLL_
//////////////////////////////////////////////////////////////////////////

#if (defined(__GNUC__ ) && !defined(__FreeBSD__) )  ||  (defined(_SOLX64))
  #define TD_USE_VA_COPY
#endif

#if defined(__GNUC__) && !defined(GCC_VERSION)
#define GCC_VERSION __GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__
#endif

#endif //#ifndef _OD_PLATFORM_SETTINGS_H_

